==> rmonitor-2.0.1.tar.bz2: rmonitor-2.0.1/resources.c <==
/*
** $Id: resources.c,v 1.6 2013/02/25 01:57:55 lpitcher Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <syslog.h>

#include "config.h"
#include "elist.h"

extern char *PgmName,	/* name of this binary, for error messages */
	    *JobScript;	/* name of JOB's target script, for message logging */

static int isValid(char *);
static int rOpen(char *);
static void rmonlog(int, const char *, ...);

/**********************************************************************/

elist *buildList(char *resources, char *interpreter, char *target)
{
  unsigned errors = 0;
  char  *resSep = ",",	/* string defining resource token separator */
	*resource;
  elist *prev = NULL,	/* temp work node for list ordering */
	*root,		/* first node of left-to-right ordered list */ 
	*head;		/* current node for list manipulation */

  /*
  ** split out individual resource names from the resources string
  ** and capture each in an elist node
  */
  if (resource = strtok(resources,resSep))
  {
    for (;resource && *resource; resource = strtok(NULL,resSep))
    {
      if (isValid(resource))
      {
        if (errors == 0)
        {
          if (head = malloc(sizeof *head))
          {
            head->ExecNext = prev; prev = head;
            if ((head->ExecTarget = strdup(resource)) == NULL)
            {
              rmonlog(SYSLOG_LEVEL_ERROR,"can't record resource %s",resource);
              ++errors;
            }
            if ((head->ExecParm.ExecFile = rOpen(head->ExecTarget)) < 0)
            {
              rmonlog(SYSLOG_LEVEL_ERROR,"can't allocate semaphore file for resource %s",resource);
              ++errors;
            }
          }
          else
          {
            rmonlog(SYSLOG_LEVEL_ERROR,"cant allocate space for resource %s",resource);
            ++errors;
            head = prev;
          }
        }
      }
      else /* resource name contained invalid characters */
      {
        rmonlog(SYSLOG_LEVEL_ERROR,"invalid resource name %s",resource);
        ++errors;
      }
    }
  }
  else	/* no resource names in resources list */
  {
    rmonlog(SYSLOG_LEVEL_ERROR,"no resources specified");
    ++errors;
  }

  /*
  ** if no problems so far, add the interpreter & target to the list
  */
  if (errors == 0)
  {
    if (head = malloc(sizeof *head))
    {
      head->ExecNext = prev; prev = head;
      if ((head->ExecTarget = strdup(target)) == NULL)
      {
        rmonlog(SYSLOG_LEVEL_ERROR,"can't record target %s",target);
        ++errors;
      }
      if ((head->ExecParm.ExecInterp = strdup(interpreter)) == NULL)
      {
        rmonlog(SYSLOG_LEVEL_ERROR,"can't record interpreter %s",interpreter);
        ++errors;
        head->ExecParm.ExecFile = -1;	/* for list delete below */
      }
    }
    else
    {
      rmonlog(SYSLOG_LEVEL_ERROR,"can't allocate space for target %s",target);
      ++errors;
      head = prev;
    }
  }

  /*
  ** if no problems so far, reverse the list to put it in same order as source
  ** otherwise, undo each node in turn, and discard the list
  */
  root = NULL;
  if (errors == 0)	/* make list run from root to tail */
  {
    while (head)
    {
      prev = head->ExecNext;
      head->ExecNext = root;
      root = head;
      head = prev;
    }
  }
  else	/* undo list, closing semaphore files as we go */
  {
    elist *temp;
    while (temp = prev)
    {
      prev = prev->ExecNext;
      if (temp->ExecParm.ExecFile >= 0) close(temp->ExecParm.ExecFile);
      if (temp->ExecTarget != NULL)	free(temp->ExecTarget);
      free(temp);
    }
  }

  return root;
}


/**********************************************************************/


/*
** isValid()	checks to see that the given string meets
**		the (albeit arbitrary) standards for content
**		String must be
**		- between 7 and 20 characters long
**		- start with alpha, digit, or '_'
**		- consist only of alphas, digits, and/or '-' or '_'
**
** Accepts:	a pointer to a resource name string
**
** Returns:	1 if string meets requirements
**		0 if string fails requirements
*/
static int isValid(char *name)
{
  int len,
      rc = 1;

  len = name ? strlen(name) : 0;

  if (len > 6 && len < 21)
  {
    int i;

    if (name[0] != '-')
    {
      for (i = 0; rc == 1 && i < len; ++i)
      {
        if (!isalpha(name[i]) && 
            !isdigit(name[i]) && 
             name[i] != '-'   && 
             name[i] != '_')
          rc = 0;
      }
    }
    else rc = 0;
  }
  else rc = 0;

  return rc;
}

/*
** rOpen()	If necessary, creates an rmonitor lock file, then
**		opens it for reading and writing.
**
** Accepts:	a pointer to resource name
**
** Updates:	opens fd in /var/lock/rmonitor.<resourcename>.lock namespace
**
** Returns:	the fd on successful allocation/open (between 0 and FD_MAX, inclusive)
**		-1 on unsuccessful open
**		-2 on pathname allocation ERROR
**		-3 on invalid resource name
*/
static int rOpen(char *resource)
{
  int fd;
  char *path = NULL;

  if (resource && *resource)
  {
    if (path = malloc(FILENAME_MAX + strlen(resource) + 1))
    {
      if (sprintf(path,"/var/lock/rmonitor.%s.lock",resource) <= FILENAME_MAX)
        fd = open(path,O_RDWR|O_CREAT,0600);
      free(path);
    }
    else fd = -2;
  }
  else fd = -3;

  return fd;
}

/*
** rmonlog()	logs given message to both stderr and syslog
**		in a common format.
**
** Accepts:	an integer indicating the syslog level necessary
**		a char pointer to a printf() format string
**		zero or more additional arguments, depending on format string
**
** Returns:	nothing
*/
void rmonlog(int level, const char *format, ...)
{
  char *msg;
  va_list ap;
  char *condition;

  msg = malloc(2049);

  va_start(ap,format);
  vsprintf(msg,format,ap);
  va_end(ap);

  condition = (level == SYSLOG_LEVEL_ERROR ? "ERROR" : "NOTE");
  fprintf(stderr,"%s: %s %s\n",PgmName,condition,msg);
  syslog(level,"JOB %s %s: %s %s",JobScript,condition,PgmName,msg);

  free(msg);
}


/*
** Copyright Lew Pitcher (lew.pitcher@digitalfreehold.ca) February, 2013
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 as
** published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program (see the licence.txt file); if not, write to the Free
** Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
** 02110-1301 USA.
*/

==> rmonitor-2.0.3.tar.bz2: rmonitor-2.0.3/resources.c <==
/*
** $Id: resources.c,v 1.4 2020/07/11 22:12:13 lpitcher Exp $
*/

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <syslog.h>
#include <unistd.h>
#include <ctype.h>

#include "logging.h"
#include "globals.h"
#include "elist.h"

/* MSG_BUF is length of formatted data buffer for rmonlog() call */
#define MSG_BUF		1024

static int isValid(char *);
static int rOpen(char *);
static void rmonlog(int, const char *, ...);

/**********************************************************************/

elist *buildList(char *resources, char *interpreter, char *target)
{
  unsigned errors = 0;
  char  *resSep = ",",	/* string defining resource token separator */
	*resource;
  elist *prev = NULL,	/* temp work node for list ordering */
	*root,		/* first node of left-to-right ordered list */ 
	*head;		/* current node for list manipulation */

  /*
  ** A resources string of "@" bypasses the resource list build,
  ** providing a built-in no-op resource.
  **
  ** This permits scripts that do not have resource requirements to
  ** run under RMonitor and gain the benefit of asynchronous execution
  ** and standardized stdout/stderr logging without the need for a dummy
  ** or no-op resource management script.
  */
  if (strcmp("@",resources) != 0)
  {
    /*
    ** split out individual resource names from the resources string
    ** and capture each in an elist node
    */
    if (resource = strtok(resources,resSep))
    {
      for (;resource && *resource; resource = strtok(NULL,resSep))
      {
        if (isValid(resource))
        {
          if (errors == 0)
          {
            if (head = malloc(sizeof *head))
            {
              head->ExecNext = prev; prev = head;
              if ((head->ExecTarget = strdup(resource)) == NULL)
              {
                rmonlog(SYSLOG_LEVEL_ERROR,"can't record resource %s",resource);
                ++errors;
              }
              if ((head->ExecParm.ExecFile = rOpen(head->ExecTarget)) < 0)
              {
                rmonlog(SYSLOG_LEVEL_ERROR,"can't allocate semaphore file for resource %s",resource);
                ++errors;
              }
            }
            else
            {
              rmonlog(SYSLOG_LEVEL_ERROR,"cant allocate space for resource %s",resource);
              ++errors;
              head = prev;
            }
          }
        }
        else /* resource name contained invalid characters */
        {
          rmonlog(SYSLOG_LEVEL_ERROR,"invalid resource name %s",resource);
          ++errors;
        }
      }
    }
    else	/* no resource names in resources list */
    {
      rmonlog(SYSLOG_LEVEL_ERROR,"no resources specified");
      ++errors;
    }
  }

  /*
  ** if no problems so far, add the interpreter & target to the list
  */
  if (errors == 0)
  {
    if (head = malloc(sizeof *head))
    {
      head->ExecNext = prev; prev = head;
      if ((head->ExecTarget = strdup(target)) == NULL)
      {
        rmonlog(SYSLOG_LEVEL_ERROR,"can't record target %s",target);
        ++errors;
      }
      if ((head->ExecParm.ExecInterp = strdup(interpreter)) == NULL)
      {
        rmonlog(SYSLOG_LEVEL_ERROR,"can't record interpreter %s",interpreter);
        ++errors;
        head->ExecParm.ExecFile = -1;	/* for list delete below */
      }
    }
    else
    {
      rmonlog(SYSLOG_LEVEL_ERROR,"can't allocate space for target %s",target);
      ++errors;
      head = prev;
    }
  }

  /*
  ** if no problems so far, reverse the list to put it in same order as source
  ** otherwise, undo each node in turn, and discard the list
  */
  root = NULL;
  if (errors == 0)	/* make list run from root to tail */
  {
    while (head)
    {
      prev = head->ExecNext;
      head->ExecNext = root;
      root = head;
      head = prev;
    }
  }
  else	/* undo list, closing semaphore files as we go */
  {
    elist *temp;
    while (temp = prev)
    {
      prev = prev->ExecNext;
      if (temp->ExecParm.ExecFile >= 0) close(temp->ExecParm.ExecFile);
      if (temp->ExecTarget != NULL)	free(temp->ExecTarget);
      free(temp);
    }
  }

  return root;
}


/**********************************************************************/


/*
** isValid()	checks to see that the given string meets
**		the (albeit arbitrary) standards for content
**		String must be
**		- between 7 and 20 characters long
**		- start with alpha, digit, or '_'
**		- consist only of alphas, digits, and/or '-' or '_'
**
** Accepts:	a pointer to a resource name string
**
** Returns:	1 if string meets requirements
**		0 if string fails requirements
*/
static int isValid(char *name)
{
  int len,
      rc = 1;

  len = name ? strlen(name) : 0;

  if (len > 6 && len < 21)
  {
    int i;

    if (name[0] != '-')
    {
      for (i = 0; rc == 1 && i < len; ++i)
      {
        if (!isalpha(name[i]) && 
            !isdigit(name[i]) && 
             name[i] != '-'   && 
             name[i] != '_')
          rc = 0;
      }
    }
    else rc = 0;
  }
  else rc = 0;

  return rc;
}

/*
** rOpen()	If necessary, creates an rmonitor lock file, then
**		opens it for reading and writing.
**
** Accepts:	a pointer to resource name
**
** Updates:	opens fd in /var/lock/rmonitor/<resourcename>.lock namespace
**
** Returns:	the fd on successful allocation/open (between 0 and FD_MAX, inclusive)
**		-1 on unsuccessful open
**		-2 on pathname allocation ERROR
**		-3 on invalid resource name
*/
static int rOpen(char *resource)
{
  int fd;
  char *path = NULL;

  if (resource && *resource)
  {
    if (path = malloc(FILENAME_MAX + strlen(resource) + 1))
    {
      if (sprintf(path,"/var/lock/rmonitor/%s.lock",resource) <= FILENAME_MAX)
        fd = open(path,O_RDWR|O_CREAT,0600);
      free(path);
    }
    else fd = -2;
  }
  else fd = -3;

  return fd;
}

/*
** rmonlog()	logs given message to both stderr and syslog
**		in a common format.
**
** Accepts:	an integer indicating the syslog level necessary
**		a char pointer to a printf() format string
**		zero or more additional arguments, depending on format string
**
** Returns:	nothing
*/
void rmonlog(int level, const char *format, ...)
{
  va_list ap;
  char *condition,
       *ellipsis,
       msg[MSG_BUF];

  va_start(ap,format);
  ellipsis = vsnprintf(msg,sizeof(msg),format,ap) >= sizeof(msg) ? "..." : "";
  va_end(ap);

  condition = (level == SYSLOG_LEVEL_ERROR ? "ERROR" : "NOTE");
  fprintf(stderr,"%s: %s %s%s\n",PgmName,condition,msg,ellipsis);
  syslog(level,"JOB %s %s: %s %s%s",JobScript,condition,PgmName,msg,ellipsis);
}


/*
** Copyright Lew Pitcher (lew.pitcher@digitalfreehold.ca) February, 2013
**
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License version 2 as
** published by the Free Software Foundation.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License along
** with this program (see the licence.txt file); if not, write to the Free
** Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
** 02110-1301 USA.
*/
